@@ -9,7 +9,14 @@ if [[ -z "$version" ]]; then
     exit 1
 fi
 
+date=$(echo "$first_line" | cut -d"(" -f2 | cut -d')' -f1)
+if [[ ! "$date" =~ ^[0-9]{4}-[0-9]{2}-[0-9]{2}$ ]]; then
+    echo "Date must set to format yyyy-mm-dd in the first line <$first_line> of the CHANGELOG"
+    exit 1
+fi
+
 echo "version=$version"
+echo "date=$date"
 changelog=$(bin/release/changelog_extract.py "$version" | grep '^\* ')
 issues=$(bin/release/get_release_note.py "$version" | grep '^\* ')
 
