@@ -5,20 +5,19 @@ from option import Option
 """
 Generate options for man
 """
-from typing import *
 import sys
 
 
-def generate_man(options: List[Option]):
+def generate_man(options: list[Option]) -> str:
     s = ""
     for option in options:
-        if not option.deprecated:
+        if not option.deprecated and not option.experimental:
             s += generate_man_option(option)
             s += "\n\n"
     return s
 
 
-def generate_man_option(option: Option):
+def generate_man_option(option: Option) -> str:
     s = "###"
     if option.short:
         s += " -%s," % option.short
@@ -28,13 +27,16 @@ def generate_man_option(option: Option):
     s += " {#%s}" % option.long.replace(".", "")
     s += "\n\n"
     s += option.description
+    if option.cli_only:
+        s += "\n\n"
+        s += "This is a cli-only option."
     return s
 
 
 def main():
     # Parse all options file given at the command line
     if len(sys.argv) < 2:
-        print("usage: generate_nab.py OPTION_FILE1 OPTION_FILE2 ...")
+        print("usage: generate_man.py OPTION_FILE1 OPTION_FILE2 ...")
         sys.exit(1)
     options = sorted(
         [Option.parse_file(filename) for filename in sys.argv[1:]],
