@@ -16,17 +16,17 @@
  *
  */
 use std::collections::HashMap;
-use std::time::Duration;
 
 use hurl_core::ast::*;
 
 use crate::http;
 use crate::http::ClientOptions;
-use crate::runner::error::Error;
+use crate::runner::cache::BodyCache;
+use crate::runner::error::RunnerError;
 use crate::runner::result::{AssertResult, EntryResult};
 use crate::runner::runner_options::RunnerOptions;
 use crate::runner::value::Value;
-use crate::runner::{request, response, CaptureResult};
+use crate::runner::{request, response, CaptureResult, RunnerErrorKind};
 use crate::util::logger::{Logger, Verbosity};
 
 /// Runs an `entry` with `http_client` and returns one [`EntryResult`].
@@ -46,6 +46,8 @@ pub fn run(
     let compressed = runner_options.compressed;
     let source_info = entry.source_info();
     let context_dir = &runner_options.context_dir;
+
+    // Evaluates our source requests given our set of variables
     let http_request = match request::eval_request(&entry.request, variables, context_dir) {
         Ok(r) => r,
         Err(error) => {
@@ -88,7 +90,8 @@ pub fn run(
             let start = entry.request.url.source_info.start;
             let end = entry.request.url.source_info.end;
             let error_source_info = SourceInfo::new(start, end);
-            let error = Error::new(error_source_info, http_error.into(), false);
+            let error =
+                RunnerError::new(error_source_info, RunnerErrorKind::Http(http_error), false);
             return EntryResult {
                 entry_index,
                 source_info,
@@ -99,20 +102,18 @@ pub fn run(
         }
     };
 
-    // We runs capture and asserts on the last HTTP request/response chains.
+    // Now, we can compute capture and asserts on the last HTTP request/response chains.
     let call = calls.last().unwrap();
     let http_response = &call.response;
-    // `time_in_ms` represent the network time of calls, not including assert processing.
-    let time_in_ms = calls
-        .iter()
-        .map(|call| call.timings.total)
-        .sum::<Duration>()
-        .as_millis();
+
+    // `transfer_duration` represent the network time of calls, not including assert processing.
+    let transfer_duration = calls.iter().map(|call| call.timings.total).sum();
 
     // We proceed asserts and captures in this order:
     // 1. first, check implicit assert on status and version. If KO, test is failed
     // 2. then, we compute captures, we might need them in asserts
     // 3. finally, run the remaining asserts
+    let mut cache = BodyCache::new();
     let mut asserts = vec![];
 
     if !runner_options.ignore_asserts {
@@ -130,7 +131,7 @@ pub fn run(
                     captures: vec![],
                     asserts,
                     errors,
-                    time_in_ms,
+                    transfer_duration,
                     compressed,
                 };
             }
@@ -140,7 +141,7 @@ pub fn run(
     let captures = match &entry.response {
         None => vec![],
         Some(response_spec) => {
-            match response::eval_captures(response_spec, http_response, variables) {
+            match response::eval_captures(response_spec, http_response, &mut cache, variables) {
                 Ok(captures) => captures,
                 Err(e) => {
                     return EntryResult {
@@ -150,7 +151,7 @@ pub fn run(
                         captures: vec![],
                         asserts,
                         errors: vec![e],
-                        time_in_ms,
+                        transfer_duration,
                         compressed,
                     };
                 }
@@ -163,8 +164,13 @@ pub fn run(
     // Compute asserts
     if !runner_options.ignore_asserts {
         if let Some(response_spec) = &entry.response {
-            let mut other_asserts =
-                response::eval_asserts(response_spec, variables, http_response, context_dir);
+            let mut other_asserts = response::eval_asserts(
+                response_spec,
+                variables,
+                http_response,
+                &mut cache,
+                context_dir,
+            );
             asserts.append(&mut other_asserts);
         }
     };
@@ -178,20 +184,22 @@ pub fn run(
         captures,
         asserts,
         errors,
-        time_in_ms,
+        transfer_duration,
         compressed,
     }
 }
 
-/// Converts a list of [`AssertResult`] to a list of [`Error`].
-fn asserts_to_errors(asserts: &[AssertResult]) -> Vec<Error> {
+/// Converts a list of [`AssertResult`] to a list of [`RunnerError`].
+fn asserts_to_errors(asserts: &[AssertResult]) -> Vec<RunnerError> {
     asserts
         .iter()
         .filter_map(|assert| assert.error())
         .map(
-            |Error {
-                 source_info, inner, ..
-             }| Error::new(source_info, inner, true),
+            |RunnerError {
+                 source_info,
+                 kind: inner,
+                 ..
+             }| RunnerError::new(source_info, inner, true),
         )
         .collect()
 }
@@ -221,7 +229,6 @@ impl ClientOptions {
             no_proxy: runner_options.no_proxy.clone(),
             insecure: runner_options.insecure,
             resolves: runner_options.resolves.clone(),
-            retry: runner_options.retry,
             ssl_no_revoke: runner_options.ssl_no_revoke,
             timeout: runner_options.timeout,
             unix_socket: runner_options.unix_socket.clone(),
@@ -246,7 +253,7 @@ fn log_request(
 ) {
     logger.debug("");
     logger.debug_important("Cookie store:");
-    for cookie in &http_client.get_cookie_storage() {
+    for cookie in &http_client.cookie_storage() {
         logger.debug(&cookie.to_string());
     }
 
