@@ -18,11 +18,10 @@
 use std::collections::HashMap;
 
 use chrono::NaiveDateTime;
-
 use hurl_core::ast::{SourceInfo, Template};
 
 use crate::runner::template::eval_template;
-use crate::runner::{Error, RunnerError, Value};
+use crate::runner::{RunnerError, RunnerErrorKind, Value};
 
 pub fn eval_to_date(
     value: &Value,
@@ -30,7 +29,7 @@ pub fn eval_to_date(
     variables: &HashMap<String, Value>,
     source_info: SourceInfo,
     assert: bool,
-) -> Result<Option<Value>, Error> {
+) -> Result<Option<Value>, RunnerError> {
     let fmt = eval_template(fmt, variables)?;
 
     match value {
@@ -39,13 +38,13 @@ pub fn eval_to_date(
                 v.and_local_timezone(chrono::Utc).unwrap(),
             ))),
             Err(_) => {
-                let inner = RunnerError::FilterInvalidInput(value.display());
-                Err(Error::new(source_info, inner, assert))
+                let kind = RunnerErrorKind::FilterInvalidInput(value.display());
+                Err(RunnerError::new(source_info, kind, assert))
             }
         },
         v => {
-            let inner = RunnerError::FilterInvalidInput(v.display());
-            Err(Error::new(source_info, inner, assert))
+            let kind = RunnerErrorKind::FilterInvalidInput(v.display());
+            Err(RunnerError::new(source_info, kind, assert))
         }
     }
 }
@@ -53,13 +52,14 @@ pub fn eval_to_date(
 #[cfg(test)]
 pub mod tests {
 
+    use std::collections::HashMap;
+
+    use chrono::{DateTime, NaiveDate, Utc};
+    use hurl_core::ast::{Filter, FilterValue, SourceInfo, Template, TemplateElement, Whitespace};
+    use hurl_core::reader::Pos;
+
     use crate::runner::filter::eval::eval_filter;
     use crate::runner::Value;
-    use chrono::{DateTime, NaiveDate, Utc};
-    use hurl_core::ast::{
-        Filter, FilterValue, Pos, SourceInfo, Template, TemplateElement, Whitespace,
-    };
-    use std::collections::HashMap;
 
     #[test]
     pub fn eval_filter_to_date() {
