@@ -18,13 +18,12 @@
 
 //! A runner for Hurl files. If you want to execute an Hurl file, this is the right place.
 
-pub use self::error::{Error, RunnerError};
+pub use self::error::{RunnerError, RunnerErrorKind};
 #[doc(hidden)]
 pub use self::event::EventListener;
 pub use self::hurl_file::run;
 #[doc(hidden)]
 pub use self::hurl_file::run_entries;
-pub use self::input::Input;
 pub use self::number::Number;
 pub use self::output::Output;
 pub use self::result::{AssertResult, CaptureResult, EntryResult, HurlResult};
@@ -33,14 +32,15 @@ pub use self::value::Value;
 
 mod assert;
 mod body;
+mod cache;
 mod capture;
+mod diff;
 mod entry;
 mod error;
 mod event;
 mod expr;
 mod filter;
 mod hurl_file;
-mod input;
 mod json;
 mod multiline;
 mod multipart;
@@ -49,7 +49,6 @@ mod options;
 mod output;
 mod predicate;
 mod predicate_value;
-mod progress;
 mod query;
 mod regex;
 mod request;
