@@ -19,13 +19,14 @@ use std::cmp::Ordering;
 use std::collections::HashMap;
 
 use hurl_core::ast::*;
+use hurl_core::reader::Pos;
 
-use crate::runner::error::Error;
+use crate::runner::error::RunnerError;
 use crate::runner::predicate_value::{eval_predicate_value, eval_predicate_value_template};
 use crate::runner::result::PredicateResult;
 use crate::runner::template::eval_template;
 use crate::runner::value::Value;
-use crate::runner::{Number, RunnerError};
+use crate::runner::{Number, RunnerErrorKind};
 use crate::util::path::ContextDir;
 
 /// Evaluates a `predicate` against an actual `value`.
@@ -68,26 +69,26 @@ pub fn eval_predicate(
     if assert_result.type_mismatch {
         let not = if predicate.not { "not " } else { "" };
         let expected = format!("{}{}", not, assert_result.expected);
-        let inner = RunnerError::AssertFailure {
+        let kind = RunnerErrorKind::AssertFailure {
             actual: assert_result.actual,
             expected,
             type_mismatch: true,
         };
-        Err(Error::new(source_info, inner, true))
+        Err(RunnerError::new(source_info, kind, true))
     } else if predicate.not && assert_result.success {
-        let inner = RunnerError::AssertFailure {
+        let kind = RunnerErrorKind::AssertFailure {
             actual: assert_result.actual,
             expected: format!("not {}", assert_result.expected),
             type_mismatch: false,
         };
-        Err(Error::new(source_info, inner, true))
+        Err(RunnerError::new(source_info, kind, true))
     } else if !predicate.not && !assert_result.success {
-        let inner = RunnerError::AssertFailure {
+        let kind = RunnerErrorKind::AssertFailure {
             actual: assert_result.actual,
             expected: assert_result.expected,
             type_mismatch: false,
         };
-        Err(Error::new(source_info, inner, true))
+        Err(RunnerError::new(source_info, kind, true))
     } else {
         Ok(())
     }
@@ -137,7 +138,7 @@ impl Number {
 }
 
 impl Value {
-    fn expected(&self) -> String {
+    pub(crate) fn format(&self) -> String {
         match self {
             Value::Bool(value) => format!("bool <{value}>"),
             Value::Bytes(values) => format!("list of size {}", values.len()),
@@ -178,27 +179,27 @@ fn expected_no_value(
     predicate_func_value: &PredicateFuncValue,
     variables: &HashMap<String, Value>,
     context_dir: &ContextDir,
-) -> Result<String, Error> {
+) -> Result<String, RunnerError> {
     match &predicate_func_value {
         PredicateFuncValue::Equal { value, .. } | PredicateFuncValue::NotEqual { value, .. } => {
             let value = eval_predicate_value(value, variables, context_dir)?;
-            Ok(value.expected())
+            Ok(value.format())
         }
         PredicateFuncValue::GreaterThan { value, .. } => {
             let value = eval_predicate_value(value, variables, context_dir)?;
-            Ok(format!("greater than <{}>", value.expected()))
+            Ok(format!("greater than <{}>", value.format()))
         }
         PredicateFuncValue::GreaterThanOrEqual { value, .. } => {
             let value = eval_predicate_value(value, variables, context_dir)?;
-            Ok(format!("greater than or equals to <{}>", value.expected()))
+            Ok(format!("greater than or equals to <{}>", value.format()))
         }
         PredicateFuncValue::LessThan { value, .. } => {
             let value = eval_predicate_value(value, variables, context_dir)?;
-            Ok(format!("less than <{}>", value.expected()))
+            Ok(format!("less than <{}>", value.format()))
         }
         PredicateFuncValue::LessThanOrEqual { value, .. } => {
             let value = eval_predicate_value(value, variables, context_dir)?;
-            Ok(format!("less than or equals to <{}>", value.expected()))
+            Ok(format!("less than or equals to <{}>", value.format()))
         }
         PredicateFuncValue::StartWith {
             value: expected, ..
@@ -220,7 +221,7 @@ fn expected_no_value(
         }
         PredicateFuncValue::Include { value, .. } => {
             let value = eval_predicate_value(value, variables, context_dir)?;
-            Ok(format!("include {}", value.expected()))
+            Ok(format!("include {}", value.format()))
         }
         PredicateFuncValue::Match {
             value: expected, ..
@@ -249,7 +250,7 @@ fn eval_predicate_func(
     variables: &HashMap<String, Value>,
     value: Option<&Value>,
     context_dir: &ContextDir,
-) -> Result<AssertResult, Error> {
+) -> Result<AssertResult, RunnerError> {
     let value = match value {
         Some(value) => value,
         None => {
@@ -316,7 +317,7 @@ fn eval_equal(
     variables: &HashMap<String, Value>,
     actual: &Value,
     context_dir: &ContextDir,
-) -> Result<AssertResult, Error> {
+) -> Result<AssertResult, RunnerError> {
     let expected = eval_predicate_value(expected, variables, context_dir)?;
     Ok(assert_values_equal(actual, &expected))
 }
@@ -327,7 +328,7 @@ fn eval_not_equal(
     variables: &HashMap<String, Value>,
     actual: &Value,
     context_dir: &ContextDir,
-) -> Result<AssertResult, Error> {
+) -> Result<AssertResult, RunnerError> {
     let expected = eval_predicate_value(expected, variables, context_dir)?;
     Ok(assert_values_not_equal(actual, &expected))
 }
@@ -338,7 +339,7 @@ fn eval_greater_than(
     variables: &HashMap<String, Value>,
     actual: &Value,
     context_dir: &ContextDir,
-) -> Result<AssertResult, Error> {
+) -> Result<AssertResult, RunnerError> {
     let expected = eval_predicate_value(expected, variables, context_dir)?;
     Ok(assert_values_greater(actual, &expected))
 }
@@ -349,7 +350,7 @@ fn eval_greater_than_or_equal(
     variables: &HashMap<String, Value>,
     actual: &Value,
     context_dir: &ContextDir,
-) -> Result<AssertResult, Error> {
+) -> Result<AssertResult, RunnerError> {
     let expected = eval_predicate_value(expected, variables, context_dir)?;
     Ok(assert_values_greater_or_equal(actual, &expected))
 }
@@ -360,7 +361,7 @@ fn eval_less_than(
     variables: &HashMap<String, Value>,
     actual: &Value,
     context_dir: &ContextDir,
-) -> Result<AssertResult, Error> {
+) -> Result<AssertResult, RunnerError> {
     let expected = eval_predicate_value(expected, variables, context_dir)?;
     Ok(assert_values_less(actual, &expected))
 }
@@ -371,7 +372,7 @@ fn eval_less_than_or_equal(
     variables: &HashMap<String, Value>,
     actual: &Value,
     context_dir: &ContextDir,
-) -> Result<AssertResult, Error> {
+) -> Result<AssertResult, RunnerError> {
     let expected = eval_predicate_value(expected, variables, context_dir)?;
     Ok(assert_values_less_or_equal(actual, &expected))
 }
@@ -383,7 +384,7 @@ fn eval_start_with(
     variables: &HashMap<String, Value>,
     actual: &Value,
     context_dir: &ContextDir,
-) -> Result<AssertResult, Error> {
+) -> Result<AssertResult, RunnerError> {
     let expected = eval_predicate_value(expected, variables, context_dir)?;
     let expected_display = format!("starts with {}", expected.display());
     let actual_display = actual.display();
@@ -416,7 +417,7 @@ fn eval_end_with(
     variables: &HashMap<String, Value>,
     actual: &Value,
     context_dir: &ContextDir,
-) -> Result<AssertResult, Error> {
+) -> Result<AssertResult, RunnerError> {
     let expected = eval_predicate_value(expected, variables, context_dir)?;
     let expected_display = format!("ends with {}", expected.display());
     let actual_display = actual.display();
@@ -449,7 +450,7 @@ fn eval_contain(
     variables: &HashMap<String, Value>,
     actual: &Value,
     context_dir: &ContextDir,
-) -> Result<AssertResult, Error> {
+) -> Result<AssertResult, RunnerError> {
     let expected = eval_predicate_value(expected, variables, context_dir)?;
     let expected_display = format!("contains {}", expected.display());
     let actual_display = actual.display();
@@ -482,7 +483,7 @@ fn eval_include(
     variables: &HashMap<String, Value>,
     actual: &Value,
     context_dir: &ContextDir,
-) -> Result<AssertResult, Error> {
+) -> Result<AssertResult, RunnerError> {
     let expected = eval_predicate_value(expected, variables, context_dir)?;
     Ok(assert_include(actual, &expected))
 }
@@ -493,13 +494,19 @@ fn eval_match(
     source_info: SourceInfo,
     variables: &HashMap<String, Value>,
     actual: &Value,
-) -> Result<AssertResult, Error> {
+) -> Result<AssertResult, RunnerError> {
     let regex = match expected {
         PredicateValue::String(template) => {
             let expected = eval_template(template, variables)?;
             match regex::Regex::new(expected.as_str()) {
                 Ok(re) => re,
-                Err(_) => return Err(Error::new(source_info, RunnerError::InvalidRegex, false)),
+                Err(_) => {
+                    return Err(RunnerError::new(
+                        source_info,
+                        RunnerErrorKind::InvalidRegex,
+                        false,
+                    ))
+                }
             }
         }
         PredicateValue::Regex(regex) => regex.inner.clone(),
@@ -524,7 +531,7 @@ fn eval_match(
 }
 
 /// Evaluates if an `actual` value is an integer.
-fn eval_is_integer(actual: &Value) -> Result<AssertResult, Error> {
+fn eval_is_integer(actual: &Value) -> Result<AssertResult, RunnerError> {
     Ok(AssertResult {
         success: matches!(actual, Value::Number(Number::Integer(_)))
             || matches!(actual, Value::Number(Number::BigInteger(_))),
@@ -535,7 +542,7 @@ fn eval_is_integer(actual: &Value) -> Result<AssertResult, Error> {
 }
 
 /// Evaluates if an `actual` value is a float.
-fn eval_is_float(actual: &Value) -> Result<AssertResult, Error> {
+fn eval_is_float(actual: &Value) -> Result<AssertResult, RunnerError> {
     Ok(AssertResult {
         success: matches!(actual, Value::Number(Number::Float(_))),
         actual: actual.display(),
@@ -545,7 +552,7 @@ fn eval_is_float(actual: &Value) -> Result<AssertResult, Error> {
 }
 
 /// Evaluates if an `actual` value is a boolean.
-fn eval_is_boolean(actual: &Value) -> Result<AssertResult, Error> {
+fn eval_is_boolean(actual: &Value) -> Result<AssertResult, RunnerError> {
     Ok(AssertResult {
         success: matches!(actual, Value::Bool(_)),
         actual: actual.display(),
@@ -555,7 +562,7 @@ fn eval_is_boolean(actual: &Value) -> Result<AssertResult, Error> {
 }
 
 /// Evaluates if an `actual` value is a string.
-fn eval_is_string(actual: &Value) -> Result<AssertResult, Error> {
+fn eval_is_string(actual: &Value) -> Result<AssertResult, RunnerError> {
     Ok(AssertResult {
         success: matches!(actual, Value::String(_)),
         actual: actual.display(),
@@ -565,7 +572,7 @@ fn eval_is_string(actual: &Value) -> Result<AssertResult, Error> {
 }
 
 /// Evaluates if an `actual` value is a collection.
-fn eval_is_collection(actual: &Value) -> Result<AssertResult, Error> {
+fn eval_is_collection(actual: &Value) -> Result<AssertResult, RunnerError> {
     Ok(AssertResult {
         success: matches!(actual, Value::Bytes(_))
             || matches!(actual, Value::List(_))
@@ -578,7 +585,7 @@ fn eval_is_collection(actual: &Value) -> Result<AssertResult, Error> {
 }
 
 /// Evaluates if an `actual` value is a date.
-fn eval_is_date(actual: &Value) -> Result<AssertResult, Error> {
+fn eval_is_date(actual: &Value) -> Result<AssertResult, RunnerError> {
     Ok(AssertResult {
         success: matches!(actual, Value::Date(_)),
         actual: actual.display(),
@@ -591,7 +598,7 @@ fn eval_is_date(actual: &Value) -> Result<AssertResult, Error> {
 ///
 /// [`eval_is_date`] performs type check (is the input of [`Value::Date`]), whereas [`eval_is_iso_date`]
 /// checks if a string conforms to a certain date-time format.
-fn eval_is_iso_date(actual: &Value) -> Result<AssertResult, Error> {
+fn eval_is_iso_date(actual: &Value) -> Result<AssertResult, RunnerError> {
     match actual {
         Value::String(actual) => Ok(AssertResult {
             success: chrono::DateTime::parse_from_rfc3339(actual).is_ok(),
@@ -609,7 +616,7 @@ fn eval_is_iso_date(actual: &Value) -> Result<AssertResult, Error> {
 }
 
 /// Evaluates if an `actual` value exists.
-fn eval_exist(actual: &Value) -> Result<AssertResult, Error> {
+fn eval_exist(actual: &Value) -> Result<AssertResult, RunnerError> {
     let actual_display = actual.display();
     let expected_display = "something".to_string();
     match actual {
@@ -629,7 +636,7 @@ fn eval_exist(actual: &Value) -> Result<AssertResult, Error> {
 }
 
 /// Evaluates if an `actual` is empty.
-fn eval_is_empty(actual: &Value) -> Result<AssertResult, Error> {
+fn eval_is_empty(actual: &Value) -> Result<AssertResult, RunnerError> {
     let expected_display = "count equals to 0".to_string();
     match actual {
         Value::List(values) => Ok(AssertResult {
@@ -672,7 +679,7 @@ fn eval_is_empty(actual: &Value) -> Result<AssertResult, Error> {
 }
 
 /// Evaluates if an `actual` value is a number.
-fn eval_is_number(actual: &Value) -> Result<AssertResult, Error> {
+fn eval_is_number(actual: &Value) -> Result<AssertResult, RunnerError> {
     Ok(AssertResult {
         success: matches!(actual, Value::Number(_)),
         actual: actual.display(),
@@ -721,6 +728,12 @@ fn assert_values_equal(actual: &Value, expected: &Value) -> AssertResult {
             expected: expected_display,
             type_mismatch: false,
         },
+        (Value::Date(value1), Value::Date(value2)) => AssertResult {
+            success: value1 == value2,
+            actual: actual_display,
+            expected: expected_display,
+            type_mismatch: false,
+        },
         // FIXME: why case (UNIT UNIT) is not treated?
         (Value::Unit, _) => AssertResult {
             success: false,
@@ -778,6 +791,12 @@ fn assert_values_not_equal(actual: &Value, expected: &Value) -> AssertResult {
             expected: expected_display,
             type_mismatch: false,
         },
+        (Value::Date(value1), Value::Date(value2)) => AssertResult {
+            success: value1 != value2,
+            actual: actual_display,
+            expected: expected_display,
+            type_mismatch: false,
+        },
         (Value::Unit, _) => AssertResult {
             success: false,
             actual: actual_display,
@@ -917,9 +936,10 @@ fn contains(haystack: &[u8], needle: &[u8]) -> bool {
 
 #[cfg(test)]
 mod tests {
-    use super::{AssertResult, *};
     use std::path::Path;
 
+    use super::{AssertResult, *};
+
     fn whitespace() -> Whitespace {
         Whitespace {
             value: String::from(" "),
@@ -979,8 +999,8 @@ mod tests {
         )
         .unwrap_err();
         assert_eq!(
-            error.inner,
-            RunnerError::AssertFailure {
+            error.kind,
+            RunnerErrorKind::AssertFailure {
                 actual: "int <10>".to_string(),
                 expected: "not int <10>".to_string(),
                 type_mismatch: false,
@@ -1256,8 +1276,8 @@ mod tests {
         let value = Value::String(String::from("http://localhost:8000"));
         let error = eval_equal(&expected, &variables, &value, &context_dir).unwrap_err();
         assert_eq!(
-            error.inner,
-            RunnerError::TemplateVariableNotDefined {
+            error.kind,
+            RunnerErrorKind::TemplateVariableNotDefined {
                 name: String::from("base_url")
             }
         );
@@ -1456,8 +1476,8 @@ mod tests {
         )
         .unwrap_err();
         assert_eq!(
-            error.inner,
-            RunnerError::AssertFailure {
+            error.kind,
+            RunnerErrorKind::AssertFailure {
                 actual: "int <1>".to_string(),
                 expected: "not starts with string <toto>".to_string(),
                 type_mismatch: true,
@@ -1513,8 +1533,8 @@ mod tests {
             .err()
             .unwrap();
         assert_eq!(
-            error.inner,
-            RunnerError::AssertFailure {
+            error.kind,
+            RunnerErrorKind::AssertFailure {
                 actual: "none".to_string(),
                 expected: "null".to_string(),
                 type_mismatch: false,
