@@ -19,14 +19,14 @@ use std::collections::HashMap;
 
 use hurl_core::ast::*;
 
-use crate::runner::error::{Error, RunnerError};
+use crate::runner::error::{RunnerError, RunnerErrorKind};
 use crate::runner::Value;
 
 /// Renders to string a `template` given a map of variables.
 pub fn eval_template(
     template: &Template,
     variables: &HashMap<String, Value>,
-) -> Result<String, Error> {
+) -> Result<String, RunnerError> {
     let Template { elements, .. } = template;
     let mut value = String::new();
     for elem in elements {
@@ -41,36 +41,42 @@ pub fn eval_template(
 fn eval_template_element(
     template_element: &TemplateElement,
     variables: &HashMap<String, Value>,
-) -> Result<String, Error> {
+) -> Result<String, RunnerError> {
     match template_element {
         TemplateElement::String { value, .. } => Ok(value.clone()),
         TemplateElement::Expression(expr) => render_expression(expr, variables),
     }
 }
 
-pub fn render_expression(expr: &Expr, variables: &HashMap<String, Value>) -> Result<String, Error> {
+pub fn render_expression(
+    expr: &Expr,
+    variables: &HashMap<String, Value>,
+) -> Result<String, RunnerError> {
     let source_info = expr.variable.source_info;
     let name = &expr.variable.name;
     let value = eval_expression(expr, variables)?;
     if value.is_renderable() {
         Ok(value.to_string())
     } else {
-        let inner = RunnerError::UnrenderableVariable {
+        let kind = RunnerErrorKind::UnrenderableVariable {
             name: name.to_string(),
             value: value.to_string(),
         };
-        Err(Error::new(source_info, inner, false))
+        Err(RunnerError::new(source_info, kind, false))
     }
 }
 
-pub fn eval_expression(expr: &Expr, variables: &HashMap<String, Value>) -> Result<Value, Error> {
+pub fn eval_expression(
+    expr: &Expr,
+    variables: &HashMap<String, Value>,
+) -> Result<Value, RunnerError> {
     let source_info = expr.variable.source_info;
     let name = &expr.variable.name;
     match variables.get(name.as_str()) {
         Some(value) => Ok(value.clone()),
         _ => {
-            let inner = RunnerError::TemplateVariableNotDefined { name: name.clone() };
-            Err(Error::new(source_info, inner, false))
+            let kind = RunnerErrorKind::TemplateVariableNotDefined { name: name.clone() };
+            Err(RunnerError::new(source_info, kind, false))
         }
     }
 }
@@ -87,6 +93,7 @@ impl Value {
 #[cfg(test)]
 mod tests {
     use hurl_core::ast::SourceInfo;
+    use hurl_core::reader::Pos;
 
     use super::*;
     use crate::runner::Number;
@@ -150,8 +157,8 @@ mod tests {
             SourceInfo::new(Pos::new(1, 3), Pos::new(1, 7))
         );
         assert_eq!(
-            error.inner,
-            RunnerError::UnrenderableVariable {
+            error.kind,
+            RunnerErrorKind::UnrenderableVariable {
                 name: "name".to_string(),
                 value: "[1,2]".to_string()
             }
@@ -164,7 +171,7 @@ mod tests {
         variables.insert("status".to_string(), Value::Bool(true));
         let expr = Expr {
             space0: Whitespace {
-                value: "".to_string(),
+                value: String::new(),
                 source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
             variable: Variable {
@@ -172,7 +179,7 @@ mod tests {
                 source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
             space1: Whitespace {
-                value: "".to_string(),
+                value: String::new(),
                 source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
             },
         };
