@@ -16,6 +16,8 @@
  *
  */
 use crate::ast::json;
+use crate::reader::Pos;
+use crate::typing::{Count, Duration};
 
 ///
 /// Hurl AST
@@ -123,23 +125,23 @@ pub struct Response {
 
 impl Response {
     /// Returns the captures list of this spec response.
-    pub fn captures(&self) -> Vec<Capture> {
+    pub fn captures(&self) -> &[Capture] {
         for section in self.sections.iter() {
             if let SectionValue::Captures(captures) = &section.value {
-                return captures.clone();
+                return captures;
             }
         }
-        vec![]
+        &[]
     }
 
     /// Returns the asserts list of this spec response.
-    pub fn asserts(&self) -> Vec<Assert> {
+    pub fn asserts(&self) -> &[Assert] {
         for section in self.sections.iter() {
             if let SectionValue::Asserts(asserts) = &section.value {
-                return asserts.clone();
+                return asserts;
             }
         }
-        vec![]
+        &[]
     }
 }
 
@@ -388,7 +390,7 @@ impl CookieAttributeName {
     }
 }
 
-#[derive(Clone, Debug, PartialEq, Eq)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
 pub enum CertificateAttributeName {
     Subject,
     Issuer,
@@ -500,32 +502,41 @@ pub enum PredicateFuncValue {
 // Primitives
 //
 #[derive(Clone, Debug, PartialEq, Eq)]
-pub enum MultilineString {
-    // FIXME: temporary type until we implement oneline as `foo` instead of ```foo```
-    OneLineText(Template),
+pub struct MultilineString {
+    pub kind: MultilineStringKind,
+    pub attributes: Vec<MultilineStringAttribute>,
+}
+
+#[derive(Clone, Debug, PartialEq, Eq)]
+pub enum MultilineStringKind {
     Text(Text),
     Json(Text),
     Xml(Text),
     GraphQl(GraphQl),
 }
 
+#[derive(Clone, Debug, PartialEq, Eq)]
+pub enum MultilineStringAttribute {
+    Escape,
+    NoVariable,
+}
+
 impl MultilineString {
     pub fn lang(&self) -> &'static str {
-        match self {
-            MultilineString::OneLineText(_) | MultilineString::Text(_) => "",
-            MultilineString::Json(_) => "json",
-            MultilineString::Xml(_) => "xml",
-            MultilineString::GraphQl(_) => "graphql",
+        match self.kind {
+            MultilineStringKind::Text(_) => "",
+            MultilineStringKind::Json(_) => "json",
+            MultilineStringKind::Xml(_) => "xml",
+            MultilineStringKind::GraphQl(_) => "graphql",
         }
     }
 
     pub fn value(&self) -> Template {
-        match self {
-            MultilineString::OneLineText(template) => template.clone(),
-            MultilineString::Text(text)
-            | MultilineString::Json(text)
-            | MultilineString::Xml(text) => text.value.clone(),
-            MultilineString::GraphQl(text) => text.value.clone(),
+        match &self.kind {
+            MultilineStringKind::Text(text)
+            | MultilineStringKind::Json(text)
+            | MultilineStringKind::Xml(text) => text.value.clone(),
+            MultilineStringKind::GraphQl(text) => text.value.clone(),
         }
     }
 }
@@ -664,18 +675,6 @@ impl PartialEq for Regex {
 }
 impl Eq for Regex {}
 
-#[derive(Copy, Clone, Debug, PartialEq, Eq)]
-pub struct Pos {
-    pub line: usize,
-    pub column: usize,
-}
-
-impl Pos {
-    pub fn new(line: usize, column: usize) -> Pos {
-        Pos { line, column }
-    }
-}
-
 #[derive(Copy, Clone, Debug, PartialEq, Eq)]
 pub struct SourceInfo {
     pub start: Pos,
@@ -719,7 +718,7 @@ pub enum OptionKind {
     ClientKey(Template),
     Compressed(BooleanOption),
     ConnectTo(Template),
-    Delay(NaturalOption),
+    Delay(DurationOption),
     Http10(BooleanOption),
     Http11(BooleanOption),
     Http2(BooleanOption),
@@ -729,16 +728,17 @@ pub enum OptionKind {
     IpV6(BooleanOption),
     FollowLocation(BooleanOption),
     FollowLocationTrusted(BooleanOption),
-    MaxRedirect(NaturalOption),
+    MaxRedirect(CountOption),
     NetRc(BooleanOption),
     NetRcFile(Template),
     NetRcOptional(BooleanOption),
     Output(Template),
     PathAsIs(BooleanOption),
     Proxy(Template),
+    Repeat(CountOption),
     Resolve(Template),
-    Retry(RetryOption),
-    RetryInterval(NaturalOption),
+    Retry(CountOption),
+    RetryInterval(DurationOption),
     Skip(BooleanOption),
     UnixSocket(Template),
     User(Template),
@@ -773,6 +773,7 @@ impl OptionKind {
             OptionKind::Output(_) => "output",
             OptionKind::PathAsIs(_) => "path-as-is",
             OptionKind::Proxy(_) => "proxy",
+            OptionKind::Repeat(_) => "repeat",
             OptionKind::Resolve(_) => "resolve",
             OptionKind::Retry(_) => "retry",
             OptionKind::RetryInterval(_) => "retry-interval",
@@ -810,6 +811,7 @@ impl OptionKind {
             OptionKind::Output(filename) => filename.to_string(),
             OptionKind::PathAsIs(value) => value.to_string(),
             OptionKind::Proxy(value) => value.to_string(),
+            OptionKind::Repeat(value) => value.to_string(),
             OptionKind::Resolve(value) => value.to_string(),
             OptionKind::Retry(value) => value.to_string(),
             OptionKind::RetryInterval(value) => value.to_string(),
@@ -838,8 +840,14 @@ pub enum NaturalOption {
 }
 
 #[derive(Clone, Debug, PartialEq, Eq)]
-pub enum RetryOption {
-    Literal(Retry),
+pub enum CountOption {
+    Literal(Count),
+    Expression(Expr),
+}
+
+#[derive(Clone, Debug, PartialEq, Eq)]
+pub enum DurationOption {
+    Literal(Duration),
     Expression(Expr),
 }
 
@@ -906,6 +914,7 @@ pub enum FilterValue {
         space0: Whitespace,
         fmt: Template,
     },
+    ToFloat,
     ToInt,
     UrlDecode,
     UrlEncode,
@@ -914,10 +923,3 @@ pub enum FilterValue {
         expr: Template,
     },
 }
-
-#[derive(Clone, Debug, PartialEq, Eq, Copy)]
-pub enum Retry {
-    None,
-    Finite(usize),
-    Infinite,
-}
