@@ -18,44 +18,43 @@
 use crate::ast::*;
 use crate::parser::error::*;
 use crate::parser::primitives::try_literal;
-use crate::parser::reader::Reader;
-
 use crate::parser::ParseResult;
+use crate::reader::Reader;
 
 pub fn natural(reader: &mut Reader) -> ParseResult<u64> {
-    let start = reader.state;
+    let start = reader.cursor();
 
     if reader.is_eof() {
-        let inner = ParseError::Expecting {
+        let kind = ParseErrorKind::Expecting {
             value: String::from("natural"),
         };
-        return Err(Error::new(start.pos, true, inner));
+        return Err(ParseError::new(start.pos, true, kind));
     }
     let first_digit = reader.read().unwrap();
     if !first_digit.is_ascii_digit() {
-        let inner = ParseError::Expecting {
+        let kind = ParseErrorKind::Expecting {
             value: String::from("natural"),
         };
-        return Err(Error::new(start.pos, true, inner));
+        return Err(ParseError::new(start.pos, true, kind));
     }
 
-    let save = reader.state;
+    let save = reader.cursor();
     let s = reader.read_while(|c| c.is_ascii_digit());
 
     // if the first digit is zero, you should not have any more digits
     if first_digit == '0' && !s.is_empty() {
-        let inner = ParseError::Expecting {
+        let kind = ParseErrorKind::Expecting {
             value: String::from("natural"),
         };
-        return Err(Error::new(save.pos, false, inner));
+        return Err(ParseError::new(save.pos, false, kind));
     }
     match format!("{first_digit}{s}").parse() {
         Ok(value) => Ok(value),
         Err(_) => {
-            let inner = ParseError::Expecting {
+            let kind = ParseErrorKind::Expecting {
                 value: String::from("natural"),
             };
-            Err(Error::new(save.pos, false, inner))
+            Err(ParseError::new(save.pos, false, kind))
         }
     }
 }
@@ -70,47 +69,47 @@ pub fn integer(reader: &mut Reader) -> ParseResult<i64> {
 }
 
 pub fn number(reader: &mut Reader) -> ParseResult<Number> {
-    let start = reader.state;
+    let start = reader.cursor();
     let sign = match try_literal("-", reader) {
         Err(_) => "",
         Ok(_) => "-",
     };
     let integer_digits = reader.read_while(|c| c.is_ascii_digit());
     if integer_digits.is_empty() {
-        let inner = ParseError::Expecting {
+        let kind = ParseErrorKind::Expecting {
             value: "number".to_string(),
         };
-        return Err(Error::new(reader.state.pos, true, inner));
+        return Err(ParseError::new(reader.cursor().pos, true, kind));
 
         // if the first digit is zero, you should not have any more digits
     } else if integer_digits.len() > 1 && integer_digits.starts_with('0') {
-        let save = reader.state;
-        let inner = ParseError::Expecting {
+        let save = reader.cursor();
+        let kind = ParseErrorKind::Expecting {
             value: String::from("natural"),
         };
-        return Err(Error::new(save.pos, false, inner));
+        return Err(ParseError::new(save.pos, false, kind));
     }
 
     // Float
     if try_literal(".", reader).is_ok() {
-        let save = reader.state;
+        let save = reader.cursor();
         let decimal_digits = reader.read_while(|c| c.is_ascii_digit());
         if decimal_digits.is_empty() {
-            let inner = ParseError::Expecting {
+            let kind = ParseErrorKind::Expecting {
                 value: String::from("decimal digits"),
             };
-            return Err(Error::new(save.pos, false, inner));
+            return Err(ParseError::new(save.pos, false, kind));
         }
         match format!("{sign}{integer_digits}.{decimal_digits}").parse() {
             Ok(value) => {
-                let encoded = reader.peek_back(start.cursor);
+                let encoded = reader.read_from(start.index);
                 Ok(Number::Float(Float { value, encoded }))
             }
             Err(_) => {
-                let inner = ParseError::Expecting {
+                let kind = ParseErrorKind::Expecting {
                     value: String::from("float"),
                 };
-                Err(Error::new(start.pos, false, inner))
+                Err(ParseError::new(start.pos, false, kind))
             }
         }
 
@@ -126,17 +125,17 @@ pub fn number(reader: &mut Reader) -> ParseResult<Number> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::ast::Pos;
+    use crate::reader::Pos;
 
     #[test]
     fn test_natural() {
         let mut reader = Reader::new("0");
         assert_eq!(natural(&mut reader).unwrap(), 0);
-        assert_eq!(reader.state.cursor, 1);
+        assert_eq!(reader.cursor().index, 1);
 
         let mut reader = Reader::new("10x");
         assert_eq!(natural(&mut reader).unwrap(), 10);
-        assert_eq!(reader.state.cursor, 2);
+        assert_eq!(reader.cursor().index, 2);
     }
 
     #[test]
@@ -145,8 +144,8 @@ mod tests {
         let error = natural(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
         assert_eq!(
-            error.inner,
-            ParseError::Expecting {
+            error.kind,
+            ParseErrorKind::Expecting {
                 value: String::from("natural")
             }
         );
@@ -156,8 +155,8 @@ mod tests {
         let error = natural(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 2 });
         assert_eq!(
-            error.inner,
-            ParseError::Expecting {
+            error.kind,
+            ParseErrorKind::Expecting {
                 value: String::from("natural")
             }
         );
@@ -167,8 +166,8 @@ mod tests {
         let error = natural(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
         assert_eq!(
-            error.inner,
-            ParseError::Expecting {
+            error.kind,
+            ParseErrorKind::Expecting {
                 value: String::from("natural")
             }
         );
@@ -179,23 +178,23 @@ mod tests {
     fn test_integer() {
         let mut reader = Reader::new("0");
         assert_eq!(integer(&mut reader).unwrap(), 0);
-        assert_eq!(reader.state.cursor, 1);
+        assert_eq!(reader.cursor().index, 1);
 
         let mut reader = Reader::new("-1");
         assert_eq!(integer(&mut reader).unwrap(), -1);
-        assert_eq!(reader.state.cursor, 2);
+        assert_eq!(reader.cursor().index, 2);
 
         let mut reader = Reader::new("0");
         assert_eq!(number(&mut reader).unwrap(), Number::Integer(0));
-        assert_eq!(reader.state.cursor, 1);
+        assert_eq!(reader.cursor().index, 1);
 
         let mut reader = Reader::new("10x");
         assert_eq!(number(&mut reader).unwrap(), Number::Integer(10));
-        assert_eq!(reader.state.cursor, 2);
+        assert_eq!(reader.cursor().index, 2);
 
         let mut reader = Reader::new("-10x");
         assert_eq!(number(&mut reader).unwrap(), Number::Integer(-10));
-        assert_eq!(reader.state.cursor, 3);
+        assert_eq!(reader.cursor().index, 3);
     }
 
     #[test]
@@ -208,7 +207,7 @@ mod tests {
                 encoded: "1.0".to_string()
             })
         );
-        assert_eq!(reader.state.cursor, 3);
+        assert_eq!(reader.cursor().index, 3);
 
         let mut reader = Reader::new("-1.0");
         assert_eq!(
@@ -218,7 +217,7 @@ mod tests {
                 encoded: "-1.0".to_string()
             })
         );
-        assert_eq!(reader.state.cursor, 4);
+        assert_eq!(reader.cursor().index, 4);
 
         let mut reader = Reader::new("1.1");
         assert_eq!(
@@ -228,7 +227,7 @@ mod tests {
                 encoded: "1.1".to_string()
             })
         );
-        assert_eq!(reader.state.cursor, 3);
+        assert_eq!(reader.cursor().index, 3);
 
         let mut reader = Reader::new("1.100");
         assert_eq!(
@@ -238,7 +237,7 @@ mod tests {
                 encoded: "1.100".to_string()
             })
         );
-        assert_eq!(reader.state.cursor, 5);
+        assert_eq!(reader.cursor().index, 5);
 
         let mut reader = Reader::new("1.01");
         assert_eq!(
@@ -248,7 +247,7 @@ mod tests {
                 encoded: "1.01".to_string()
             })
         );
-        assert_eq!(reader.state.cursor, 4);
+        assert_eq!(reader.cursor().index, 4);
 
         let mut reader = Reader::new("1.010");
         assert_eq!(
@@ -258,7 +257,7 @@ mod tests {
                 encoded: "1.010".to_string()
             })
         );
-        assert_eq!(reader.state.cursor, 5);
+        assert_eq!(reader.cursor().index, 5);
 
         // provide more digits than necessary
         let mut reader = Reader::new("-0.3333333333333333333");
@@ -269,7 +268,7 @@ mod tests {
                 encoded: "-0.3333333333333333333".to_string()
             })
         );
-        assert_eq!(reader.state.cursor, 22);
+        assert_eq!(reader.cursor().index, 22);
 
         let mut reader = Reader::new("1000000000000000000000.5");
         assert_eq!(
@@ -279,7 +278,7 @@ mod tests {
                 encoded: "1000000000000000000000.5".to_string()
             })
         );
-        assert_eq!(reader.state.cursor, 24);
+        assert_eq!(reader.cursor().index, 24);
     }
 
     #[test]
@@ -287,8 +286,8 @@ mod tests {
         let mut reader = Reader::new("");
         let error = number(&mut reader).err().unwrap();
         assert_eq!(
-            error.inner,
-            ParseError::Expecting {
+            error.kind,
+            ParseErrorKind::Expecting {
                 value: String::from("number")
             }
         );
@@ -298,8 +297,8 @@ mod tests {
         let mut reader = Reader::new("-");
         let error = number(&mut reader).err().unwrap();
         assert_eq!(
-            error.inner,
-            ParseError::Expecting {
+            error.kind,
+            ParseErrorKind::Expecting {
                 value: String::from("number")
             }
         );
@@ -310,8 +309,8 @@ mod tests {
         let error = number(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
         assert_eq!(
-            error.inner,
-            ParseError::Expecting {
+            error.kind,
+            ParseErrorKind::Expecting {
                 value: String::from("number")
             }
         );
@@ -321,8 +320,8 @@ mod tests {
         let error = number(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 3 });
         assert_eq!(
-            error.inner,
-            ParseError::Expecting {
+            error.kind,
+            ParseErrorKind::Expecting {
                 value: String::from("decimal digits")
             }
         );
