@@ -16,12 +16,12 @@
  *
  */
 use crate::ast::*;
-use crate::parser::combinators::*;
+use crate::combinator::choice;
 use crate::parser::error::*;
 use crate::parser::predicate_value::predicate_value;
 use crate::parser::primitives::*;
-use crate::parser::reader::Reader;
 use crate::parser::ParseResult;
+use crate::reader::Reader;
 
 pub fn predicate(reader: &mut Reader) -> ParseResult<Predicate> {
     let (not, space0) = predicate_not(reader);
@@ -35,7 +35,7 @@ pub fn predicate(reader: &mut Reader) -> ParseResult<Predicate> {
 
 // can not fail
 fn predicate_not(reader: &mut Reader) -> (bool, Whitespace) {
-    let save = reader.state;
+    let save = reader.cursor();
     let no_whitespace = Whitespace {
         value: String::new(),
         source_info: SourceInfo {
@@ -47,7 +47,7 @@ fn predicate_not(reader: &mut Reader) -> (bool, Whitespace) {
         match one_or_more_spaces(reader) {
             Ok(space) => (true, space),
             Err(_) => {
-                reader.state = save;
+                reader.seek(save);
                 (false, no_whitespace)
             }
         }
@@ -57,17 +57,17 @@ fn predicate_not(reader: &mut Reader) -> (bool, Whitespace) {
 }
 
 fn predicate_func(reader: &mut Reader) -> ParseResult<PredicateFunc> {
-    let start = reader.state.pos;
+    let start = reader.cursor();
     let value = predicate_func_value(reader)?;
-    let end = reader.state.pos;
+    let end = reader.cursor();
     Ok(PredicateFunc {
-        source_info: SourceInfo { start, end },
+        source_info: SourceInfo::new(start.pos, end.pos),
         value,
     })
 }
 
 fn predicate_func_value(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
-    let start = reader.state;
+    let start = reader.cursor();
     match choice(
         &[
             equal_predicate,
@@ -94,9 +94,9 @@ fn predicate_func_value(reader: &mut Reader) -> ParseResult<PredicateFuncValue>
         ],
         reader,
     ) {
-        Err(Error {
+        Err(ParseError {
             recoverable: true, ..
-        }) => Err(Error::new(start.pos, false, ParseError::Predicate)),
+        }) => Err(ParseError::new(start.pos, false, ParseErrorKind::Predicate)),
         x => x,
     }
 }
@@ -164,7 +164,7 @@ fn greater_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     } else {
         one_or_more_spaces(reader)?
     };
-    let start = reader.state;
+    let start = reader.cursor();
     let value = predicate_value(reader)?;
     if value.is_number() || value.is_string() || value.is_expression() {
         Ok(PredicateFuncValue::GreaterThan {
@@ -173,7 +173,11 @@ fn greater_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
             operator,
         })
     } else {
-        Err(Error::new(start.pos, false, ParseError::PredicateValue))
+        Err(ParseError::new(
+            start.pos,
+            false,
+            ParseErrorKind::PredicateValue,
+        ))
     }
 }
 
@@ -187,7 +191,7 @@ fn greater_or_equal_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncV
     } else {
         one_or_more_spaces(reader)?
     };
-    let start = reader.state;
+    let start = reader.cursor();
     let value = predicate_value(reader)?;
     if value.is_number() || value.is_string() || value.is_expression() {
         Ok(PredicateFuncValue::GreaterThanOrEqual {
@@ -196,7 +200,11 @@ fn greater_or_equal_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncV
             operator,
         })
     } else {
-        Err(Error::new(start.pos, false, ParseError::PredicateValue))
+        Err(ParseError::new(
+            start.pos,
+            false,
+            ParseErrorKind::PredicateValue,
+        ))
     }
 }
 
@@ -210,7 +218,7 @@ fn less_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     } else {
         one_or_more_spaces(reader)?
     };
-    let start = reader.state;
+    let start = reader.cursor();
     let value = predicate_value(reader)?;
     if value.is_number() || value.is_string() || value.is_expression() {
         Ok(PredicateFuncValue::LessThan {
@@ -219,7 +227,11 @@ fn less_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
             operator,
         })
     } else {
-        Err(Error::new(start.pos, false, ParseError::PredicateValue))
+        Err(ParseError::new(
+            start.pos,
+            false,
+            ParseErrorKind::PredicateValue,
+        ))
     }
 }
 
@@ -233,7 +245,7 @@ fn less_or_equal_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValu
     } else {
         one_or_more_spaces(reader)?
     };
-    let start = reader.state;
+    let start = reader.cursor();
     let value = predicate_value(reader)?;
     if value.is_number() || value.is_string() || value.is_expression() {
         Ok(PredicateFuncValue::LessThanOrEqual {
@@ -242,17 +254,25 @@ fn less_or_equal_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValu
             operator,
         })
     } else {
-        Err(Error::new(start.pos, false, ParseError::PredicateValue))
+        Err(ParseError::new(
+            start.pos,
+            false,
+            ParseErrorKind::PredicateValue,
+        ))
     }
 }
 
 fn start_with_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("startsWith", reader)?;
     let space0 = one_or_more_spaces(reader)?;
-    let save = reader.state;
+    let save = reader.cursor();
     let value = predicate_value(reader)?;
     if !value.is_string() && !value.is_bytearray() {
-        return Err(Error::new(save.pos, false, ParseError::PredicateValue));
+        return Err(ParseError::new(
+            save.pos,
+            false,
+            ParseErrorKind::PredicateValue,
+        ));
     }
     Ok(PredicateFuncValue::StartWith { space0, value })
 }
@@ -260,10 +280,14 @@ fn start_with_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue>
 fn end_with_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("endsWith", reader)?;
     let space0 = one_or_more_spaces(reader)?;
-    let save = reader.state;
+    let save = reader.cursor();
     let value = predicate_value(reader)?;
     if !value.is_string() && !value.is_bytearray() {
-        return Err(Error::new(save.pos, false, ParseError::PredicateValue));
+        return Err(ParseError::new(
+            save.pos,
+            false,
+            ParseErrorKind::PredicateValue,
+        ));
     }
     Ok(PredicateFuncValue::EndWith { space0, value })
 }
@@ -271,10 +295,14 @@ fn end_with_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
 fn contain_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("contains", reader)?;
     let space0 = one_or_more_spaces(reader)?;
-    let save = reader.state;
+    let save = reader.cursor();
     let value = predicate_value(reader)?;
     if !value.is_string() && !value.is_bytearray() {
-        return Err(Error::new(save.pos, false, ParseError::PredicateValue));
+        return Err(ParseError::new(
+            save.pos,
+            false,
+            ParseErrorKind::PredicateValue,
+        ));
     }
     Ok(PredicateFuncValue::Contain { space0, value })
 }
@@ -289,10 +317,14 @@ fn include_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
 fn match_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
     try_literal("matches", reader)?;
     let space0 = one_or_more_spaces(reader)?;
-    let save = reader.state;
+    let save = reader.cursor();
     let value = predicate_value(reader)?;
     if !matches!(value, PredicateValue::String(_)) && !matches!(value, PredicateValue::Regex(_)) {
-        return Err(Error::new(save.pos, false, ParseError::PredicateValue));
+        return Err(ParseError::new(
+            save.pos,
+            false,
+            ParseErrorKind::PredicateValue,
+        ));
     }
     Ok(PredicateFuncValue::Match { space0, value })
 }
@@ -350,7 +382,7 @@ fn is_number_predicate(reader: &mut Reader) -> ParseResult<PredicateFuncValue> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::ast::Pos;
+    use crate::reader::Pos;
 
     #[test]
     fn test_predicate_not() {
@@ -365,7 +397,7 @@ mod tests {
                 }
             )
         );
-        assert_eq!(reader.state.pos, Pos { line: 1, column: 1 });
+        assert_eq!(reader.cursor().pos, Pos { line: 1, column: 1 });
 
         let mut reader = Reader::new("not XX");
         assert_eq!(
@@ -378,7 +410,7 @@ mod tests {
                 }
             )
         );
-        assert_eq!(reader.state.pos, Pos { line: 1, column: 5 });
+        assert_eq!(reader.cursor().pos, Pos { line: 1, column: 5 });
     }
 
     #[test]
@@ -413,7 +445,7 @@ mod tests {
         let error = predicate_func(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
         assert!(!error.recoverable);
-        assert_eq!(error.inner, ParseError::Predicate);
+        assert_eq!(error.kind, ParseErrorKind::Predicate);
     }
 
     #[test]
@@ -523,7 +555,7 @@ mod tests {
             }
         );
         assert!(!error.recoverable);
-        assert_eq!(error.inner, ParseError::PredicateValue);
+        assert_eq!(error.kind, ParseErrorKind::PredicateValue);
     }
 
     #[test]
