@@ -16,15 +16,15 @@
  *
  */
 use crate::ast::*;
-use crate::parser::combinators::*;
+use crate::combinator::{optional, recover, zero_or_more};
 use crate::parser::error::*;
 use crate::parser::filter::filters;
 use crate::parser::predicate::predicate;
 use crate::parser::primitives::*;
 use crate::parser::query::query;
-use crate::parser::reader::Reader;
 use crate::parser::string::*;
 use crate::parser::{filename, key_string, option, ParseResult};
+use crate::reader::{Pos, Reader};
 
 pub fn request_sections(reader: &mut Reader) -> ParseResult<Vec<Section>> {
     let sections = zero_or_more(request_section, reader)?;
@@ -39,12 +39,10 @@ pub fn response_sections(reader: &mut Reader) -> ParseResult<Vec<Section>> {
 fn request_section(reader: &mut Reader) -> ParseResult<Section> {
     let line_terminators = optional_line_terminators(reader)?;
     let space0 = zero_or_more_spaces(reader)?;
-    let start = reader.state.pos;
+    let start = reader.cursor();
     let name = section_name(reader)?;
-    let source_info = SourceInfo {
-        start,
-        end: reader.state.pos,
-    };
+    let source_info = SourceInfo::new(start.pos, reader.cursor().pos);
+
     let line_terminator0 = line_terminator(reader)?;
     let value = match name.as_str() {
         "QueryStringParams" => section_value_query_params(reader)?,
@@ -54,9 +52,9 @@ fn request_section(reader: &mut Reader) -> ParseResult<Section> {
         "Cookies" => section_value_cookies(reader)?,
         "Options" => section_value_options(reader)?,
         _ => {
-            let inner = ParseError::RequestSectionName { name: name.clone() };
-            let pos = Pos::new(start.line, start.column + 1);
-            return Err(Error::new(pos, false, inner));
+            let kind = ParseErrorKind::RequestSectionName { name: name.clone() };
+            let pos = Pos::new(start.pos.line, start.pos.column + 1);
+            return Err(ParseError::new(pos, false, kind));
         }
     };
 
@@ -72,20 +70,19 @@ fn request_section(reader: &mut Reader) -> ParseResult<Section> {
 fn response_section(reader: &mut Reader) -> ParseResult<Section> {
     let line_terminators = optional_line_terminators(reader)?;
     let space0 = zero_or_more_spaces(reader)?;
-    let start = reader.state.pos;
+    let start = reader.cursor();
     let name = section_name(reader)?;
-    let source_info = SourceInfo {
-        start,
-        end: reader.state.pos,
-    };
+    let end = reader.cursor();
+    let source_info = SourceInfo::new(start.pos, end.pos);
+
     let line_terminator0 = line_terminator(reader)?;
     let value = match name.as_str() {
         "Captures" => section_value_captures(reader)?,
         "Asserts" => section_value_asserts(reader)?,
         _ => {
-            let inner = ParseError::ResponseSectionName { name: name.clone() };
-            let pos = Pos::new(start.line, start.column + 1);
-            return Err(Error::new(pos, false, inner));
+            let kind = ParseErrorKind::ResponseSectionName { name: name.clone() };
+            let pos = Pos::new(start.pos.line, start.pos.column + 1);
+            return Err(ParseError::new(pos, false, kind));
         }
     };
 
@@ -99,15 +96,15 @@ fn response_section(reader: &mut Reader) -> ParseResult<Section> {
 }
 
 fn section_name(reader: &mut Reader) -> ParseResult<String> {
-    let pos = reader.state.pos;
+    let pos = reader.cursor().pos;
     try_literal("[", reader)?;
     let name = reader.read_while(|c| c.is_alphanumeric());
     if name.is_empty() {
         // Could be the empty json array for the body
-        let inner = ParseError::Expecting {
+        let kind = ParseErrorKind::Expecting {
             value: "a valid section name".to_string(),
         };
-        return Err(Error::new(pos, true, inner));
+        return Err(ParseError::new(pos, true, kind));
     }
     try_literal("]", reader)?;
     Ok(name)
@@ -175,12 +172,12 @@ fn cookie(reader: &mut Reader) -> ParseResult<Cookie> {
 }
 
 fn multipart_param(reader: &mut Reader) -> ParseResult<MultipartParam> {
-    let save = reader.state;
+    let save = reader.cursor();
     match file_param(reader) {
         Ok(f) => Ok(MultipartParam::FileParam(f)),
         Err(e) => {
             if e.recoverable {
-                reader.state = save;
+                reader.seek(save);
                 let param = key_value(reader)?;
                 Ok(MultipartParam::Param(param))
             } else {
@@ -216,10 +213,10 @@ fn file_value(reader: &mut Reader) -> ParseResult<FileValue> {
     let f = filename::parse(reader)?;
     let space1 = zero_or_more_spaces(reader)?;
     literal(";", reader)?;
-    let save = reader.state;
+    let save = reader.cursor();
     let (space2, content_type) = match line_terminator(reader) {
         Ok(_) => {
-            reader.state = save;
+            reader.seek(save);
             let space2 = Whitespace {
                 value: String::new(),
                 source_info: SourceInfo {
@@ -230,7 +227,7 @@ fn file_value(reader: &mut Reader) -> ParseResult<FileValue> {
             (space2, None)
         }
         Err(_) => {
-            reader.state = save;
+            reader.seek(save);
             let space2 = zero_or_more_spaces(reader)?;
             let content_type = file_content_type(reader)?;
             (space2, Some(content_type))
@@ -247,16 +244,16 @@ fn file_value(reader: &mut Reader) -> ParseResult<FileValue> {
 }
 
 fn file_content_type(reader: &mut Reader) -> ParseResult<String> {
-    let start = reader.state;
+    let start = reader.cursor();
     let mut buf = String::new();
     let mut spaces = String::new();
-    let mut save = reader.state;
+    let mut save = reader.cursor();
     while let Some(c) = reader.read() {
         if c.is_alphanumeric() || c == '/' || c == ';' || c == '=' || c == '-' {
             buf.push_str(spaces.as_str());
             spaces = String::new();
             buf.push(c);
-            save = reader.state;
+            save = reader.cursor();
         } else if c == ' ' {
             spaces.push(' ');
         } else {
@@ -264,9 +261,13 @@ fn file_content_type(reader: &mut Reader) -> ParseResult<String> {
         }
     }
 
-    reader.state = save;
+    reader.seek(save);
     if buf.is_empty() {
-        return Err(Error::new(start.pos, false, ParseError::FileContentType));
+        return Err(ParseError::new(
+            start.pos,
+            false,
+            ParseErrorKind::FileContentType,
+        ));
     }
     Ok(buf)
 }
@@ -316,7 +317,6 @@ fn assert(reader: &mut Reader) -> ParseResult<Assert> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::ast::Pos;
 
     #[test]
     fn test_section_name() {
@@ -426,8 +426,8 @@ mod tests {
         let error = response_section(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 1 });
         assert_eq!(
-            error.inner,
-            ParseError::Expecting {
+            error.kind,
+            ParseErrorKind::Expecting {
                 value: String::from("[")
             }
         );
@@ -437,8 +437,8 @@ mod tests {
         let error = response_section(&mut reader).err().unwrap();
         assert_eq!(error.pos, Pos { line: 1, column: 2 });
         assert_eq!(
-            error.inner,
-            ParseError::ResponseSectionName {
+            error.kind,
+            ParseErrorKind::ResponseSectionName {
                 name: String::from("Assertsx")
             }
         );
@@ -476,8 +476,8 @@ mod tests {
         );
         assert!(!error.recoverable);
         assert_eq!(
-            error.inner,
-            ParseError::Expecting {
+            error.kind,
+            ParseErrorKind::Expecting {
                 value: "}}".to_string()
             }
         );
@@ -548,21 +548,21 @@ mod tests {
             file_content_type(&mut reader).unwrap(),
             "text/html".to_string()
         );
-        assert_eq!(reader.state.cursor, 9);
+        assert_eq!(reader.cursor().index, 9);
 
         let mut reader = Reader::new("text/plain; charset=us-ascii");
         assert_eq!(
             file_content_type(&mut reader).unwrap(),
             "text/plain; charset=us-ascii".to_string()
         );
-        assert_eq!(reader.state.cursor, 28);
+        assert_eq!(reader.cursor().index, 28);
 
         let mut reader = Reader::new("text/html # comment");
         assert_eq!(
             file_content_type(&mut reader).unwrap(),
             "text/html".to_string()
         );
-        assert_eq!(reader.state.cursor, 9);
+        assert_eq!(reader.cursor().index, 9);
     }
 
     #[test]
@@ -628,7 +628,7 @@ mod tests {
                 },
             }
         );
-        assert_eq!(reader.state.cursor, 43);
+        assert_eq!(reader.cursor().index, 43);
     }
 
     #[test]
@@ -643,8 +643,8 @@ mod tests {
             }
         );
         assert_eq!(
-            error.inner,
-            ParseError::Expecting {
+            error.kind,
+            ParseErrorKind::Expecting {
                 value: "\" or /".to_string()
             }
         );
@@ -660,8 +660,8 @@ mod tests {
             }
         );
         assert_eq!(
-            error.inner,
-            ParseError::Expecting {
+            error.kind,
+            ParseErrorKind::Expecting {
                 value: "line_terminator".to_string()
             }
         );
@@ -790,7 +790,7 @@ mod tests {
                 source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 12)),
             }
         );
-        assert_eq!(reader.state.pos, Pos { line: 3, column: 1 });
+        assert_eq!(reader.cursor().pos, Pos { line: 3, column: 1 });
 
         let mut reader = Reader::new("[BasicAuth]\nHTTP 200\n");
         assert_eq!(
@@ -816,6 +816,6 @@ mod tests {
                 source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 12)),
             }
         );
-        assert_eq!(reader.state.pos, Pos { line: 2, column: 1 });
+        assert_eq!(reader.cursor().pos, Pos { line: 2, column: 1 });
     }
 }
